@@ -25,7 +25,7 @@
 
 methods:
           classical inflect
-          pl plnoun plverb pladj no num a an
+          pl plnoun plverb pladj sinoun no num a an
           plequal plnounequal plverbequal pladjequal
           prespart
           ordinal
@@ -34,11 +34,11 @@ methods:
           defnoun defverb defadj defa defan
 
     INFLECTIONS:    classical inflect
-          pl plnoun plverb pladj plequal
+          pl plnoun plverb pladj sinoun plequal
           no num a an prespart
 
     PLURALS:   classical inflect
-          pl plnoun plverb pladj no num
+          pl plnoun plverb pladj sinoun no num
           plequal plnounequal plverbequal pladjequal
 
     COMPARISONS:    classical 
@@ -74,6 +74,14 @@ class NumOutOfRangeError(Exception): pass
 class BadUserDefinedPatternError(Exception): pass
 class BadRcFileError(Exception): pass
 
+__ver_major__ = 0
+__ver_minor__ = 1
+__ver_patch__ = 6
+__ver_sub__ = ""
+__version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,
+                              __ver_patch__,__ver_sub__)
+
+
 STDOUT_ON = False
 
 def print3(txt):
@@ -146,12 +154,31 @@ pl_sb_irregular = {
     'blouse'      : 'blouses',
     'Rom'         : 'Roma',
     'rom'         : 'roma',
+    'carmen'      : 'carmina',
 }
 
 pl_sb_irregular.update(pl_sb_irregular_s)
-
 pl_sb_irregular_keys = enclose('|'.join(pl_sb_irregular.keys()))
 
+si_sb_irregular = dict([(v, k) for (k, v) in pl_sb_irregular.iteritems()])
+keys = si_sb_irregular.keys()
+for k in keys:
+    if '|' in k:
+        k1, k2 = k.split('|')
+        si_sb_irregular[k1] = si_sb_irregular[k2] = si_sb_irregular[k]
+        del si_sb_irregular[k]
+        
+si_sb_irregular_keys = enclose('|'.join(si_sb_irregular.keys()))
+
+# Z's that don't double
+
+pl_sb_z_zes = (
+    "quartz", "topaz", "snooz(?=e)",
+)
+
+pl_sb_z_zes = enclose('|'.join(pl_sb_z_zes))
+
+
 # CLASSICAL "..is" -> "..ides"
 
 pl_sb_C_is_ides = [
@@ -198,7 +225,7 @@ pl_sb_C_a_ae = enclose('|'.join ((
 # CLASSICAL "..en" -> "..ina"
 
 pl_sb_C_en_ina = joinstem(-2, ((
-    "stamen", "foramen", "lumen", "carmen"
+    "stamen", "foramen", "lumen",
 )))
 
 # UNCONDITIONAL "..um" -> "..a"
@@ -473,6 +500,136 @@ pl_sb_singular_s = enclose('|'.join ([
    ] + pl_sb_C_is_ides
 ))
 
+# PLURALS ENDING IN uses -> use
+
+
+si_sb_ois_oi_case = (
+    'Bolshois', 'Hanois'
+    )
+
+si_sb_uses_use_case = (
+    'Betelgeuses', 'Duses', 'Meuses', 'Syracuses', 'Toulouses',
+    )
+
+si_sb_uses_use = (
+    'abuses', 'applauses', 'blouses',
+    'carouses', 'causes', 'chartreuses', 'clauses',
+    'contuses', 'douses', 'excuses', 'fuses', 
+    'grouses', 'hypotenuses', 'masseuses',
+    'menopauses', 'misuses', 'muses', 'overuses', 'pauses',
+    'peruses', 'profuses', 'recluses', 'reuses',
+    'ruses', 'souses', 'spouses', 'suffuses', 'transfuses', 'uses', 
+)
+
+si_sb_ies_ie_case = (
+    'Addies', 'Aggies', 'Allies', 'Amies', 'Angies', 'Annies',
+'Annmaries', 'Archies', 'Arties', 'Aussies', 'Barbies',
+'Barries', 'Basies', 'Bennies', 'Bernies', 'Berties', 'Bessies',
+'Betties', 'Billies', 'Blondies', 'Bobbies', 'Bonnies',
+'Bowies', 'Brandies', 'Bries', 'Brownies', 'Callies',
+'Carnegies', 'Carries', 'Cassies', 'Charlies', 'Cheries',
+'Christies', 'Connies', 'Curies', 'Dannies', 'Debbies', 'Dixies',
+'Dollies', 'Donnies', 'Drambuies', 'Eddies', 'Effies', 'Ellies',
+'Elsies', 'Eries', 'Ernies', 'Essies', 'Eugenies', 'Fannies',
+'Flossies', 'Frankies', 'Freddies', 'Gillespies', 'Goldies',
+'Gracies', 'Guthries', 'Hallies', 'Hatties', 'Hetties',
+'Hollies', 'Jackies', 'Jamies', 'Janies', 'Jannies', 'Jeanies',
+'Jeannies', 'Jennies', 'Jessies', 'Jimmies', 'Jodies', 'Johnies',
+'Johnnies', 'Josies', 'Julies', 'Kalgoorlies', 'Kathies', 'Katies',
+'Kellies', 'Kewpies', 'Kristies', 'Laramies', 'Lassies', 'Lauries',
+'Leslies', 'Lessies', 'Lillies', 'Lizzies', 'Lonnies', 'Lories',
+'Lorries', 'Lotties', 'Louies', 'Mackenzies', 'Maggies', 'Maisies',
+'Mamies', 'Marcies', 'Margies', 'Maries', 'Marjories', 'Matties',
+'McKenzies', 'Melanies', 'Mickies', 'Millies', 'Minnies', 'Mollies',
+'Mounties', 'Nannies', 'Natalies', 'Nellies', 'Netties', 'Ollies',
+'Ozzies', 'Pearlies', 'Pottawatomies', 'Reggies', 'Richies', 'Rickies',
+'Robbies', 'Ronnies', 'Rosalies', 'Rosemaries', 'Rosies', 'Roxies',
+'Rushdies', 'Ruthies', 'Sadies', 'Sallies', 'Sammies', 'Scotties',
+'Selassies', 'Sherries', 'Sophies', 'Stacies', 'Stefanies', 'Stephanies',
+'Stevies', 'Susies', 'Sylvies', 'Tammies', 'Terries', 'Tessies',
+'Tommies', 'Tracies', 'Trekkies', 'Valaries', 'Valeries', 'Valkyries',
+'Vickies', 'Virgies', 'Willies', 'Winnies', 'Wylies', 'Yorkies',
+    )
+
+si_sb_ies_ie = (
+'aeries', 'baggies', 'belies', 'biggies', 'birdies', 'bogies',
+'bonnies', 'boogies', 'bookies', 'bourgeoisies', 'brownies',
+'budgies', 'caddies', 'calories', 'camaraderies', 'cockamamies',
+'collies', 'cookies', 'coolies', 'cooties', 'coteries', 'crappies',
+'curies', 'cutesies',  'dogies', 'eyrie', 'floozies', 'footsies',
+'freebies', 'genies', 'goalies', 'groupies',
+'hies', 'jalousies', 'junkies',
+'kiddies', 'laddies', 'lassies', 'lies',
+'lingeries', 'magpies', 'menageries', 'mommies', 'movies', 'neckties',
+'newbies', 'nighties', 'oldies', 'organdies', 'overlies',
+'pies', 'pinkies', 'pixies', 'potpies', 'prairies',
+'quickies', 'reveries', 'rookies', 'rotisseries', 'softies', 'sorties',
+'species', 'stymies', 'sweeties', 'ties', 'underlies', 'unties',
+'veggies', 'vies', 'yuppies', 'zombies',
+)
+
+
+si_sb_oes_oe_case = (
+    'Chloes', 'Crusoes', 'Defoes', 'Faeroes', 'Ivanhoes', 'Joes',
+    'McEnroes', 'Moes', 'Monroes', 'Noes', 'Poes', 'Roscoes',
+    'Tahoes', 'Tippecanoes', 'Zoes',
+    )
+
+si_sb_oes_oe = (
+    'aloes', 'backhoes', 'canoes',
+    'does', 'floes', 'foes', 'hoes', 'mistletoes',
+    'oboes', 'pekoes', 'roes', 'sloes',
+    'throes', 'tiptoes', 'toes', 'woes',
+)
+
+si_sb_z_zes = (
+    "quartzes", "topazes",
+)
+
+si_sb_zzes_zz = (
+'buzzes', 'fizzes', 'frizzes', 'razzes' 
+    )
+
+si_sb_ches_che_case = (
+'Andromaches', 'Apaches', 'Blanches',  'Comanches',
+'Nietzsches', 'Porsches', 'Roches',
+)
+
+si_sb_ches_che = (
+'aches', 'avalanches', 'backaches', 'bellyaches', 'caches',
+'cloches', 'creches', 'douches', 'earaches', 'fiches',
+'headaches', 'heartaches', 'microfiches',
+ 'niches', 'pastiches', 'psyches', 'quiches',
+'stomachaches', 'toothaches',
+)
+
+si_sb_xes_xe = (
+'annexes', 'axes', 'deluxes', 'pickaxes',
+    )
+
+si_sb_sses_sse_case = (
+'Hesses', 'Jesses', 'Larousses', 'Matisses',
+)
+si_sb_sses_sse = (
+'bouillabaisses', 'crevasses', 'demitasses', 'impasses',
+'mousses', 'posses',
+    )
+
+si_sb_ves_ve_case = (
+# *[nwl]ives -> [nwl]live
+'Clives', 'Palmolives',
+)
+si_sb_ves_ve = (
+# *[^d]eaves -> eave
+'interweaves', 'weaves',
+
+# *[nwl]ives -> [nwl]live
+'olives', 
+
+# *[eoa]lves -> [eoa]lve
+'bivalves', 'dissolves', 'resolves', 'salves', 'twelves', 'valves',
+    )
+
 
 plverb_special_s = enclose('|'.join (
     [pl_sb_singular_s] +
@@ -484,7 +641,7 @@ plverb_special_s = enclose('|'.join (
 ]))
 
 pl_sb_postfix_adj = {
-    'general' : ['(?!major|lieutenant|brigadier|adjutant)\S+'],
+    'general' : ['(?!major|lieutenant|brigadier|adjutant|.*star)\S+'],
     'martial' : ['court'],
 }
 
@@ -496,6 +653,26 @@ for k in pl_sb_postfix_adj.keys():
 pl_sb_postfix_adj_stems = '(' + '|'.join(pl_sb_postfix_adj.values()) + ')(.*)'
 
 
+# PLURAL WORDS ENDING IS es GO TO SINGULAR is
+
+si_sb_es_is = (
+'amanuenses', 'amniocenteses', 'analyses', 'antitheses',
+'apotheoses', 'arterioscleroses', 'atheroscleroses', 'axes',
+# 'bases', # bases -> basis
+'catalyses', 'catharses', 'chasses', 'cirrhoses',
+'cocces', 'crises', 'diagnoses', 'dialyses', 'diereses',
+'electrolyses', 'emphases', 'exegeses', 'geneses',
+'halitoses', 'hydrolyses', 'hypnoses', 'hypotheses', 'hystereses',
+'metamorphoses', 'metastases', 'misdiagnoses', 'mitoses', 
+'mononucleoses', 'narcoses', 'necroses', 'nemeses', 'neuroses',
+'oases', 'osmoses', 'osteoporoses', 'paralyses', 'parentheses',
+'parthenogeneses', 'periphrases', 'photosyntheses', 'probosces',
+'prognoses', 'prophylaxes', 'prostheses', 'preces', 'psoriases',
+'psychoanalyses', 'psychokineses', 'psychoses', 'scleroses',
+'scolioses', 'sepses', 'silicoses', 'symbioses', 'synopses',
+'syntheses', 'taxes', 'telekineses', 'theses', 'thromboses',
+'tuberculoses', 'urinalyses',
+)
 
 pl_prep = enclose('|'.join( """
     about above across after among around at athwart before behind
@@ -530,6 +707,13 @@ pl_pron_nom = {
 "theirs" : "theirs",
 }
 
+si_pron_nom = dict([(v, k) for (k, v) in pl_pron_nom.iteritems()])
+si_pron_nom['they'] = 'it'
+si_pron_nom['themselves'] = 'itself'
+si_pron_nom['theirs'] = 'its'
+si_pron_nom['we'] = 'I'
+
+
 pl_pron_acc = {
 #   ACCUSATIVE      REFLEXIVE
 
@@ -543,6 +727,12 @@ pl_pron_acc = {
 
 pl_pron_acc_keys = enclose('|'.join(pl_pron_acc.keys()))
 
+si_pron_acc = dict([(v, k) for (k, v) in pl_pron_acc.iteritems()])
+si_pron_acc['them'] = 'it'
+si_pron_acc['themselves'] = 'itself'
+si_pron_acc_keys = enclose('|'.join(si_pron_acc.keys()))
+
+
 plverb_irregular_pres = {
 #   1st PERS. SING.     2ND PERS. SING.     3RD PERS. SINGULAR
 #               3RD PERS. (INDET.)  
@@ -647,7 +837,14 @@ A_y_cons = 'y(b[lor]|cl[ea]|fere|gg|p[ios]|rou|tt)'
 A_explicit_an = enclose('|'.join((
     "euler",
     "hour(?!i)", "heir", "honest", "hono",
-    "[fhlmnrsx]-?th",
+    )))
+
+A_ordinal_an = enclose('|'.join((
+    "[aefhilmnorsx]-?th",
+    )))
+
+A_ordinal_a = enclose('|'.join((
+    "[bcdgjkpqtuvwyz]-?th",
     )))
 
 
@@ -736,6 +933,7 @@ class engine:
         self.pl_sb_user_defined = []
         self.pl_v_user_defined  = []
         self.pl_adj_user_defined  = []
+        self.si_sb_user_defined = []
         self.A_a_user_defined   = []
 
     def defnoun(self, singular, plural):
@@ -746,6 +944,7 @@ class engine:
         self.checkpat(singular)
         self.checkpatplural(plural)
         self.pl_sb_user_defined.extend((singular, plural))
+        self.si_sb_user_defined.extend((plural, singular))
         return 1
 
     def defverb(self, s1, p1, s2, p2, s3, p3):
@@ -918,6 +1117,13 @@ class engine:
         '''
         return self.pladj(matchobject.group(1), matchobject.group(3))
 
+    def sinounmo(self, matchobject):
+        '''
+        sinoun but take a matchobject
+        use groups 1 and 3 in matchobject
+        '''
+        return self.sinoun(matchobject.group(1), matchobject.group(3))
+
     def amo(self, matchobject):
         '''
         A but take a matchobject
@@ -965,7 +1171,7 @@ class engine:
         e.g. inflect('The plural of cat is pl(cat)') returns
         'The plural of cat is cats'
 
-        can use pl, plnoun, plverb, pladj, a, an, no, ordinal,
+        can use pl, plnoun, plverb, pladj, sinoun, a, an, no, ordinal,
         numwords and prespart
 
         '''
@@ -993,6 +1199,10 @@ class engine:
                         r"(?x)\bpladj \( ([^),]*) (, ([^)]*) )? \)  ",
                         self.pladjmo, section)
                     total += count
+                    (section, count) = subn(
+                        r"(?x)\bsinoun   \( ([^),]*) (, ([^)]*) )? \)  ",
+                        self.sinounmo, section)
+                    total += count
                     (section, count) = subn(
                         r"(?x)\ban?    \( ([^),]*) (, ([^)]*) )? \)  ",
                         self.amo, section)
@@ -1178,6 +1388,29 @@ class engine:
         '''
         return self._plequal(word1, word2, self.pladj)
 
+
+    def sinoun(self, text, count=None):
+        '''
+        Return the singular of text, where text is a plural noun.
+
+        If count supplied, then return the singular if count is one of:
+            1, a, an, one, each, every, this, that or if count is None
+        otherwise return text unchanged.
+        
+        Whitespace at the start and end is preserved.
+
+        '''
+        pre, word, post = self.partition_word(text)
+        if not word:
+            return text
+        sing = self._sinoun(word, count)
+        if sing != False:
+            plural = self.postprocess(word,
+                  self._sinoun(word, count))
+            return "%s%s%s" % (pre, plural, post)
+        return False
+    
+        
     def _plequal(self, word1, word2, pl):
         classval = self.classical_dict.copy()
         self.classical_dict = all_classical.copy()
@@ -1353,7 +1586,7 @@ class engine:
 
         mo = search(r"(%s)$" % pl_sb_U_man_mans, word, IGNORECASE)
         if mo:
-            return "%ss" % mo.group(1)
+            return "%ss" % word
 
         mo = search(r"(\S*)quy$", word, IGNORECASE)
         if mo:
@@ -1379,6 +1612,9 @@ class engine:
             if mo:
                 return a[1] % mo.group(1)
 
+        if lowerword == 'die':
+            return 'dice'
+
 
 # HANDLE UNASSIMILATED IMPORTS
 
@@ -1389,13 +1625,13 @@ class engine:
         for a in (
                   (r"(.*)zoon$", "%szoa"),
                   (r"(.*[csx])is$", "%ses"),
-                  (r"(%s)ch$" % pl_sb_U_ch_chs, "%schs"),
-                  (r"(%s)ex$" % pl_sb_U_ex_ices, "%sices"),
-                  (r"(%s)ix$" % pl_sb_U_ix_ices, "%sices"),
-                  (r"(%s)um$" % pl_sb_U_um_a, "%sa"),
-                  (r"(%s)us$" % pl_sb_U_us_i, "%si"),
-                  (r"(%s)on$" % pl_sb_U_on_a, "%sa"),
-                  (r"(%s)$" % pl_sb_U_a_ae, "%se"),
+                  (r"(.*%s)ch$" % pl_sb_U_ch_chs, "%schs"),
+                  (r"(.*%s)ex$" % pl_sb_U_ex_ices, "%sices"),
+                  (r"(.*%s)ix$" % pl_sb_U_ix_ices, "%sices"),
+                  (r"(.*%s)um$" % pl_sb_U_um_a, "%sa"),
+                  (r"(.*%s)us$" % pl_sb_U_us_i, "%si"),
+                  (r"(.*%s)on$" % pl_sb_U_on_a, "%sa"),
+                  (r"(.*%s)$" % pl_sb_U_a_ae, "%se"),
                  ):
             mo = search(a[0], word, IGNORECASE)
             if mo:
@@ -1444,20 +1680,25 @@ class engine:
 # caught below?
 #
 # removing it as I can't find a case that executes it
-#
+# TODO: check this again
 #
 #        if (self.classical_dict['names']):
 #            mo = search(r"([A-Z].*s)$", word)
 #            if mo:
 #                return "%ses" % mo.group(1)
 
+        mo = search(r"(%s)$" % pl_sb_z_zes, word, IGNORECASE)
+        if mo:
+            return "%ses" % mo.group(1)
+
+
         mo = search(r"^(.*[^z])(z)$", word, IGNORECASE)
         if mo:
             return "%szzes" % mo.group(1)
 
         for a in (
                   (r"^(.*)([cs]h|x|zz|ss)$",  "%s%ses"),
-#                  (r"(.*)(us)$", "%s%ses"),
+#                  (r"(.*)(us)$", "%s%ses"),  TODO: why is this commented?
                  ):
             mo = search(a[0], word, IGNORECASE)
             if mo:
@@ -1557,6 +1798,8 @@ class engine:
             return False
         if search(r"\s", word):
             return False
+        if lowerword == 'quizzes':
+            return 'quiz'
 
 
 # HANDLE STANDARD 3RD PERSON (CHOP THE ...(e)s OFF SINGLE WORDS)
@@ -1652,6 +1895,300 @@ class engine:
         return False
 
 
+    def _sinoun(self, word, count=None):
+        count = self.get_count(count)
+
+
+# DEFAULT TO PLURAL
+
+        if count == 2:
+            return word
+
+# HANDLE USER-DEFINED NOUNS
+
+        value = self.ud_match(word, self.si_sb_user_defined)
+        if value is not None:
+            return value
+
+
+# HANDLE EMPTY WORD, SINGULAR COUNT AND UNINFLECTED PLURALS
+
+        if word == '':
+            return word
+
+        lowerword = word.lower()
+
+        if word in si_sb_ois_oi_case:
+            return word[:-1]
+
+        if search(r"^%s$" % pl_sb_uninflected, word, IGNORECASE):
+            return word
+
+        if (self.classical_dict['herd'] and
+               lowerword in pl_sb_uninflected_herd):
+            return word
+
+# HANDLE COMPOUNDS ("Governor General", "mother-in-law", "aide-de-camp", ETC.)
+        mo = search(r"^(?:%s)$" % pl_sb_postfix_adj_stems, word, IGNORECASE)
+        if mo and mo.group(2) != '':
+            return "%s%s" % (self._sinoun(mo.group(1), 1), mo.group(2))
+
+        #how to reverse this one?
+        # mo = search(r"^(?:%s)$" % pl_sb_prep_dual_compound, word, IGNORECASE)
+        # if mo and mo.group(2) != '' and mo.group(3) != '':
+        #     return "%s%s%s" % (self._sinoun(mo.group(1), 1),
+        #                        mo.group(2),
+        #                        self._sinoun(mo.group(3), 1))
+
+        mo = search(r"^(?:%s)$" % pl_sb_prep_compound, word, IGNORECASE)
+        if mo and mo.group(2) != '':
+                return "%s%s" % (self._sinoun(mo.group(1), 1), mo.group(2))
+
+
+# HANDLE PRONOUNS
+
+        mo = search(r"^((?:%s)\s+)(%s)$" % (pl_prep, si_pron_acc_keys), word,
+                                                         IGNORECASE)
+        if mo:
+            return "%s%s" % (mo.group(1), si_pron_acc[mo.group(2).lower()])
+
+        try:
+            return si_pron_nom[word.lower()]
+        except KeyError:
+            pass
+
+        try:
+            return si_pron_acc[word.lower()]
+        except KeyError:
+            pass
+
+# HANDLE ISOLATED IRREGULAR PLURALS 
+
+        mo = search(r"(.*)\b(%s)$" % si_sb_irregular_keys, word, IGNORECASE)
+        if mo:
+            try:
+                return "%s%s" % (mo.group(1),
+                             si_sb_irregular[mo.group(2)])
+            except KeyError:
+                return "%s%s" % (mo.group(1),
+                             si_sb_irregular[mo.group(2).lower()])
+
+        mo = search(r"(%s)s$" % pl_sb_U_man_mans, word, IGNORECASE)
+        if mo:
+            return word[:-1]
+
+        mo = search(r"(\S*)quies$", word, IGNORECASE)
+        if mo:
+            return "%squy" % mo.group(1)
+
+        mo = search(r"(\S*)(persons)$", word, IGNORECASE)
+        if mo:
+             return "%sperson" % mo.group(1)
+
+        mo = search(r"(\S*)(people)$", word, IGNORECASE)
+        if mo:
+             return "%sperson" % mo.group(1)
+
+# HANDLE FAMILIES OF IRREGULAR PLURALS 
+
+        for a in (
+                  (r"(.*)men$", "%sman"),
+                  (r"(.*[ml])ice$", "%souse"),
+                  (r"(.*)geese$", "%sgoose"),
+                  (r"(.*)teeth$", "%stooth"),
+                  (r"(.*)feet$", "%sfoot"),
+                 ):
+            mo = search(a[0], word, IGNORECASE)
+            if mo:
+                return a[1] % mo.group(1)
+            
+        if lowerword == 'dice':
+            return 'die'
+
+
+# HANDLE UNASSIMILATED IMPORTS
+
+        if search(r"(.*)ceps$", word, IGNORECASE):
+            return word
+
+        for a in (
+                  (r"(.*)zoa$", "%szoon"),
+                  #(r"(.*[csx])es$", "%sis"), # test too general to have this early
+                  (r"(.*%s)chs$" % pl_sb_U_ch_chs, "%sch"),
+                  (r"(.*%s)ices$" % pl_sb_U_ex_ices, "%sex"),
+                  (r"(.*%s)ices$" % pl_sb_U_ix_ices, "%six"),
+                  (r"(.*%s)a$" % pl_sb_U_um_a, "%sum"),
+                  (r"(.*%s)i$" % pl_sb_U_us_i, "%sus"),
+                  (r"(.*%s)a$" % pl_sb_U_on_a, "%son"),
+                  (r"(.*%s)e$" % pl_sb_U_a_ae, "%s"),
+                 ):
+            mo = search(a[0], word, IGNORECASE)
+            if mo:
+                return a[1] % mo.group(1)
+
+# HANDLE INCOMPLETELY ASSIMILATED IMPORTS
+
+        if (self.classical_dict['ancient']):
+
+            for a in (
+                  (r"(.*)trices$", "%strix"),
+                  (r"(.*)eaux$", "%seau"),
+                  (r"(.*)ieux$", "%sieu"),
+                  (r"(.{2,}[yia])nges$", "%snx"),
+                  (r"(%s)ina$" % pl_sb_C_en_ina, "%sen"),
+                  (r"(%s)ices$" % pl_sb_C_ex_ices, "%sex"),
+                  (r"(%s)ices$" % pl_sb_C_ix_ices, "%six"),
+                  (r"(%s)a$" % pl_sb_C_um_a, "%sum"),
+                  (r"(%s)i$" % pl_sb_C_us_i, "%sus"),
+                  (r"(%s)$" % pl_sb_C_us_us, "%s"),
+                  (r"(%s)e$" % pl_sb_C_a_ae, "%s"),
+                  (r"(%s)ata$" % pl_sb_C_a_ata_stems, "%sa"),
+                  (r"(%s)ides$" % pl_sb_C_is_ides_stems, "%sis"),
+                  (r"(%s)i$" % pl_sb_C_o_i_stems, "%so"),
+                  (r"(%s)a$" % pl_sb_C_on_a, "%son"),
+                  (r"(%s)im$" % pl_sb_C_im, "%s"),
+                  (r"(%s)i$" % pl_sb_C_i, "%s"),
+                 ):
+                mo = search(a[0], word, IGNORECASE)
+                if mo:
+                    return a[1] % mo.group(1)
+
+
+# HANDLE PLURLS ENDING IN uses -> use
+
+        if (lowerword[-6:] == 'houses' or
+            word in si_sb_uses_use_case or
+            lowerword in si_sb_uses_use):
+            return word[:-1]
+
+# HANDLE PLURLS ENDING IN ies -> ie
+
+        if word in si_sb_ies_ie_case or lowerword in si_sb_ies_ie:
+            return word[:-1]
+
+
+# HANDLE PLURLS ENDING IN oes -> oe
+
+        if (lowerword[-5:] == 'shoes' or
+            word in si_sb_oes_oe_case or
+            lowerword in si_sb_oes_oe):
+            return word[:-1]
+
+# HANDLE SINGULAR NOUNS ENDING IN ...s OR OTHER SILIBANTS
+
+        if (word in si_sb_sses_sse_case or
+            lowerword in si_sb_sses_sse):
+            return word[:-1]
+
+        mo = search(r"(%s)es$" % pl_sb_singular_s, word)
+        #TODO: stop [A-Z].*es matching all words, something sublter than above line which stopping any upper case matching other words
+        #mo = search(r"(%s)es$" % pl_sb_singular_s, word, IGNORECASE)
+        if mo:
+            return "%s" % mo.group(1)
+
+
+# Wouldn't special words
+# ending with 's' always have been caught, regardless of them starting
+# with a capital letter (i.e. being names)
+# It makes sense below to do this for words ending in 'y' so that
+# Sally -> Sallys. But not sure it makes sense here. Where is the case
+# of a word ending in s that is caught here and would otherwise have been
+# caught below?
+#
+# removing it as I can't find a case that executes it
+# TODO: check this again
+#
+#        if (self.classical_dict['names']):
+#            mo = search(r"([A-Z].*ses)$", word)
+#            if mo:
+#                return "%s" % mo.group(1)
+
+        if lowerword in si_sb_z_zes:
+            return word[:-2]
+
+        if lowerword in si_sb_zzes_zz:
+            return word[:-2]
+
+        if lowerword[-4:] == 'zzes':
+            return word[:-3]
+
+        if (word in si_sb_ches_che_case or
+            lowerword in si_sb_ches_che):
+            return word[:-1]
+
+        if lowerword[-4:] in ('ches', 'shes'):
+            return word[:-2]
+
+        if lowerword in si_sb_xes_xe:
+            return word[:-1]
+
+        if lowerword[-3:] == 'xes':
+            return word[:-2]
+
+            
+#                  (r"(.*)(us)es$", "%s%s"),  TODO: why is this commented?
+
+
+# HANDLE ...f -> ...ves
+
+        if (word in si_sb_ves_ve_case or
+            lowerword in si_sb_ves_ve):
+            return word[:-1]
+
+        if word[-3:] == 'ves':
+            for a in (
+                      (r"(.*[eao])lves$", "%slf"),
+                      (r"(.*[^d])eaves$", "%seaf"),
+                      (r"(.*[nlw])ives$", "%sife"),
+                      (r"(.*)arves$", "%sarf"),
+                     ):
+                mo = search(a[0], word, IGNORECASE)
+                if mo:
+                    return a[1] % mo.group(1)
+
+# HANDLE ...y
+
+        mo = search(r"(.*[aeiou])ys$", word, IGNORECASE)
+        if mo:
+            return "%sy" % mo.group(1)
+
+        if (self.classical_dict['names']):
+            mo = search(r"([A-Z].*y)s$", word)
+            if mo:
+                return "%s" % mo.group(1)
+
+        mo = search(r"(.*)ies$", word, IGNORECASE)
+        if mo:
+            return "%sy" % mo.group(1)
+
+
+# HANDLE ...o
+
+        for a, num in (
+                  (r"(%s)s$" % pl_sb_U_o_os, -1),
+                  (r"(.*[aeiou]o)s$", -1),
+                  (r"(.*o)es$", -2),
+                 ):
+            mo = search(a, word, IGNORECASE)
+            if mo:
+                return word[:num]
+
+# UNASSIMILATED IMPORTS FINAL RULE
+
+        if word in si_sb_es_is:
+            return word[:-2] + 'is'
+
+
+# OTHERWISE JUST REMOVE ...s
+
+        if lowerword[-1] == 's':
+            return word[:-1]
+
+
+# COULD NOT FIND SINGULAR
+        return False
+
+
 # ADJECTIVES
 
     def a(self, text, count=1):
@@ -1694,6 +2231,16 @@ class engine:
         if value is not None:
             return "%s %s" % (value, word)
 
+    # HANDLE ORDINAL FORMS
+
+        for a in (
+                  (r"^(%s)" % A_ordinal_a, "a"),
+                  (r"^(%s)" % A_ordinal_an, "an"),
+                 ):
+            mo = search(a[0], word, IGNORECASE)
+            if mo:
+                return "%s %s" % (a[1], word)
+
 
 # HANDLE SPECIAL CASES
 
@@ -1807,7 +2354,10 @@ class engine:
                           (r"ue$", r"u"), #TODO: isn't ue$ -> u encompassed in the following rule?
                           (r"([auy])e$", r"\g<1>"),
                           (r"ski$", r"ski"),
-                          (r"i$", r""),
+                          (r"[^b]i$", r""),
+                          (r"^(are|were)$", r"be"),
+                          (r"^(had)$", r"hav"),
+                          (r"^(hoe)$", r"\g<1>"),
                           (r"([^e])e$", r"\g<1>"),
                           (r"er$", r"er"),
                           (r"([^aeiou][aeiouy]([bdgmnprst]))$", "\g<1>\g<2>"),
@@ -1831,7 +2381,7 @@ class engine:
         ordinal('one') returns 'first'
 
         '''
-        if match(r"\d", num):
+        if match(r"\d", str(num)):
             n = int(num)
             try:
                 post = nth[n%100]
